==> goirc-0.5.tgz: goirc-0.5/irc.go <==
// Copyright 2009 Eric Estabrooks. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// This package provides an interactive terminal client for irc
// Commands:
// /tlsconnect server:port (port is typically 994 for sirc)
// /connect server:port
// /nick new-nick
// /quit (quits the program)
// /join #channel (used to join and also to switch the active channel)
// /part #channel (leave a channel)
// /me action (send action to active channel)
// plain text gets passed as a message to the active channel
// other commands can be sent to the server by starting with a /
// followed by the raw command the server expects
package main

import "os"
import "strings"
import "regexp"
import "bytes"
import "fmt"

import "./ircclient"
import "./term"

var command *regexp.Regexp = regexp.MustCompile("([^ ]*)(.*)");

var default_nick string = "my_nick";

var topics map[string]string;

func main() {
	var _buf [4096]byte;

	// load in help topics
	topics = make(map[string]string);

	client := ircclient.New();
	client.Nick = default_nick;

	t, err := term.NewTerm(os.Stdin.Fd());

	if err != nil {
		fmt.Printf("Term failed: %v\n", err);
		os.Exit(1);
	}

	t.Clear();
	t.SetScroll(0, t.MaxRow - 2);
	t.LineWrap(true);

	go _handle_server_input(client, t);

	cmdTable := make(map[string]func(string, *ircclient.IRCClient, *term.Term));
	cmdTable["connect"] = cmdConnect;
	cmdTable["tlsconnect"] = cmdTLSConnect;
	cmdTable["channel"] = cmdChannel;
	cmdTable["join"] = cmdChannel;
	cmdTable["me"] = cmdAction;
	cmdTable["quit"] = cmdQuit;
	cmdTable["help"] = cmdHelp;
	cmdTable["resize"] = cmdResize;

	conv := new(bytes.Buffer);
	for {
		n, err := t.ReadAt(t.MaxRow-1, 0, &_buf);
		if (err != nil) {
			os.Exit(1);
		}
		if (n > 0) {
			t.WriteBytesAt(t.MaxRow-2, 0, _buf[0:n]);
			t.CursorAt(t.MaxRow-1, 0);
			conv.Reset();
			conv.Write(_buf[0:n-1]);
			input := conv.String();
			t.HistoryAdd(input);
			if (_buf[0] == '/') {
				res := command.MatchStrings(input[1:n-1]);
				if res != nil && len(res) > 0 {
					cmdFunc, ok := cmdTable[res[1]];
					if (ok) {
						cmdFunc(strings.TrimSpace(res[2]), client, t);
					} else {
						client.Write(_buf[1:n-1]);
						client.Write(strings.Bytes("\r\n"));
					}
				} else {
					client.Write(_buf[1:n-1]);
					client.Write(strings.Bytes("\r\n"));
				}
			} else {
				client.Write(strings.Bytes("PRIVMSG " + client.Channel + " :"));
				client.Write(_buf[0:n-1]);
				client.Write(strings.Bytes("\r\n"));
			}
		}
	}
}

//var notice *regexp.Regexp = regexp.MustCompile("^.+ :(.*)");
var msg *regexp.Regexp = regexp.MustCompile("^:(.+)!(.+) (.+) (.+) :(.+)");
var action *regexp.Regexp = regexp.MustCompile("ACTION(.+)");

func _handle_server_input(client* ircclient.IRCClient, t *term.Term) {
	var buff [1]byte;
	var s string;

	input := new(bytes.Buffer);

	if client == nil { return; }

	for {
		<-client.CanRead;
		n, _ := client.Read(&buff);
		for n > 0 {
			if buff[0] != '\r' && buff[0] != '\n' {
				input.WriteByte(buff[0]);
			} else {
				if input.Len() > 0 {
					res := msg.MatchStrings(input.String());
					if res != nil && len(res) > 0 {
						actiontmp := res[5];
						actionres := action.MatchStrings(actiontmp);
						if actionres != nil && len(actionres) > 0 {
							s = fmt.Sprintf("%c[31;2m%s>        *|%s%s%c[37;2m\n", 033, res[4], res[1], actionres[1], 033);
						} else {
							channel := res[4];
							if channel[0] >= 'A' {
								s = fmt.Sprintf("%c[32;1m> %8s| %s%c[37;2m\n", 033, res[1], res[5], 033);
							} else {
								s = fmt.Sprintf("%c[33;1m%s> %8s| %s%c[37;2m\n", 033, res[4], res[1], res[5], 033);
							}
						}
						t.WriteAt(t.MaxRow-2, 0, s);
					} else {
						s = fmt.Sprintf("%c[34;1m> %s%c[37;2m\n", 033, input.String(), 033);
						t.WriteAt(t.MaxRow-2, 0, s);
					}
					t.CursorAt(t.MaxRow-1, 0);
					input.Reset();
				}
			}
			n, _ = client.Read(&buff);
		}
	}
}

func cmdConnect(serverName string, client *ircclient.IRCClient, t *term.Term) {
	if client != nil {
		client.Connect(serverName, client.Nick);
	}
}

func cmdTLSConnect(serverName string, client *ircclient.IRCClient, t *term.Term) {
	if client != nil {
		client.TLSConnect(serverName, client.Nick);
	}
}

func cmdChannel(channel string, client *ircclient.IRCClient, t *term.Term) {
	if client != nil && len(channel) > 0 {
		client.SetChannel(channel);
		if channel[0] < 'A' {
			client.Write(strings.Bytes("JOIN " + channel + "\r\n"));
		}
	}
}

func cmdAction(action string, client *ircclient.IRCClient, t *term.Term) {
	if (client != nil) {
		client.Write(strings.Bytes("PRIVMSG " + client.Channel + " :ACTION " + action + "\r\n"));
	}
}

func cmdQuit(reason string, client *ircclient.IRCClient, t *term.Term) {
	if client != nil {
		client.Write(strings.Bytes("QUIT\r\n"));
		client.Quit("user is done");
	}
	if t != nil {
		t.SetFullScroll();
		t.CursorAt(t.MaxRow, 0);
		t.ClearLine(t.MaxRow);
		t.Restore();
	}
	os.Exit(0);
}

func cmdHelp(topic string, client *ircclient.IRCClient, t *term.Term) {
	if t != nil {
		help, ok := topics[topic];
		if ok {
			t.WriteAt(t.MaxRow-2, 0, help);
		} else {
			s := fmt.Sprintf("%c[32;2m%s%s%s%s%s%s%s%s%s%s%s%c[37;2m\n", 033, "help\n",
			" /tlsconnect server:port - port for sirc typically 994\n",
			" /connect server:port - port for irc typically 6666-6669\n",
			" /join channel - join an irc channel, also used to make it the current active one\n",
			" /channel channel - same as join\n",
			" /part channel - leave a channel\n",
			" /me action - send an action to active channel\n",
			" /nick new-nick - change nick to new-nick\n",
			" /quit - disconnect from irc servers and exit program\n",
			" /resize - do this after changing the terminal size\n",
			" /whatever else - gets sent as-is to the server (basically raw command)\n",
			" text without a / - gets sent as a PRIVMSG to active channel\n", 033);
			t.WriteAt(t.MaxRow-2, 0, s);
		}
	}
}

func cmdResize(notused string, client *ircclient.IRCClient, t *term.Term) {
	if t != nil {
		t.SetFullScroll();
		t.Resize();
		t.Clear();
		t.SetScroll(0, t.MaxRow - 2);
		t.CursorAt(t.MaxRow - 1, 0);
	}
}

==> goirc-0.7.tgz: goirc-0.7/irc.go <==
// Copyright 2009 Eric Estabrooks. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// This package provides an interactive terminal client for irc
// Commands:
// /tlsconnect server:port (port is typically 994 for sirc)
// /connect server:port
// /nick new-nick
// /quit (quits the program)
// /join #channel (used to join and also to switch the active channel)
// /part #channel (leave a channel)
// /me action (send action to active channel)
// plain text gets passed as a message to the active channel
// other commands can be sent to the server by starting with a /
// followed by the raw command the server expects
package main

import "os"
import "os/signal"
import "syscall"
import "strings"
import "regexp"
import "bytes"
import "fmt"

import "./ircclient"
import "./term"

var command *regexp.Regexp = regexp.MustCompile("([^ ]*)(.*)");

var default_nick string = "my_nick";

var topics map[string]string;

var needResize chan bool;

func main() {
	var _buf [4096]byte;

	// load in help topics
	topics = make(map[string]string);

	client := ircclient.New();
	client.Nick = default_nick;

	t, err := term.NewTerm(os.Stdin.Fd());

	needResize = make(chan bool);

	if err != nil {
		fmt.Printf("Term failed: %v\n", err);
		os.Exit(1);
	}

	t.Clear();
	t.SetScroll(0, t.MaxRow - 2);
	t.LineWrap(true);
	t.WriteAt(t.MaxRow-2, 0, "Welcome to goirc, type /help for command information\n");

	go _handleSignals(t);
	go _handle_server_input(client, t);

	cmdTable := make(map[string]func(string, *ircclient.IRCClient, *term.Term));
	cmdTable["connect"] = cmdConnect;
	cmdTable["tlsconnect"] = cmdTLSConnect;
	cmdTable["channel"] = cmdChannel;
	cmdTable["join"] = cmdChannel;
	cmdTable["me"] = cmdAction;
	cmdTable["quit"] = cmdQuit;
	cmdTable["help"] = cmdHelp;
	cmdTable["resize"] = cmdResize;

	conv := new(bytes.Buffer);
	for {
		select {
			case <-needResize:
				cmdResize("sigwinch", nil, t);
			default:
		}
		n, err := t.ReadAt(t.MaxRow-1, 0, &_buf);
		if (err != nil) {
			os.Exit(1);
		}
		if (n > 0) {
			t.WriteBytesAt(t.MaxRow-2, 0, _buf[0:n]);
			t.CursorAt(t.MaxRow-1, 0);
			conv.Reset();
			conv.Write(_buf[0:n-1]);
			input := conv.String();
			t.HistoryAdd(input);
			if (_buf[0] == '/') {
				res := command.MatchStrings(input[1:n-1]);
				if res != nil && len(res) > 0 {
					cmdFunc, ok := cmdTable[res[1]];
					if (ok) {
						cmdFunc(strings.TrimSpace(res[2]), client, t);
					} else {
						client.Write(_buf[1:n-1]);
						client.Write(strings.Bytes("\r\n"));
					}
				} else {
					client.Write(_buf[1:n-1]);
					client.Write(strings.Bytes("\r\n"));
				}
			} else {
				client.Write(strings.Bytes("PRIVMSG " + client.Channel + " :"));
				client.Write(_buf[0:n-1]);
				client.Write(strings.Bytes("\r\n"));
			}
		}
	}
}

func _handleSignals(t *term.Term) {
	for {
		sig := (<-signal.Incoming).(signal.UnixSignal);
		if (sig == syscall.SIGWINCH) {
			needResize <- true;
		} else {
			fmt.Printf("caught a signal %s, exiting\n", sig);
			os.Exit(0);
		}
	}
}

//var notice *regexp.Regexp = regexp.MustCompile("^.+ :(.*)");
var msg *regexp.Regexp = regexp.MustCompile("^:(.+)!(.+) (.+) (.+) :(.+)");
var action *regexp.Regexp = regexp.MustCompile("ACTION(.+)");

func _handle_server_input(client* ircclient.IRCClient, t *term.Term) {
	var buff [1]byte;
	var s string;

	input := new(bytes.Buffer);

	if client == nil { return; }

	for {
		<-client.CanRead;
		n, _ := client.Read(&buff);
		for n > 0 {
			if buff[0] != '\r' && buff[0] != '\n' {
				input.WriteByte(buff[0]);
			} else {
				if input.Len() > 0 {
					res := msg.MatchStrings(input.String());
					if res != nil && len(res) > 0 {
						actiontmp := res[5];
						actionres := action.MatchStrings(actiontmp);
						if actionres != nil && len(actionres) > 0 {
							s = fmt.Sprintf("%c[31;0m%s>        *|%s%s%c[0m\n", 033, res[4], res[1], actionres[1], 033);
						} else {
							channel := res[4];
							if channel[0] >= 'A' {
								s = fmt.Sprintf("%c[32;1m> %8s| %s%c[0m\n", 033, res[1], res[5], 033);
							} else {
								s = fmt.Sprintf("%c[33;1m%s> %8s| %s%c[0m\n", 033, res[4], res[1], res[5], 033);
							}
						}
						t.WriteAt(t.MaxRow-2, 0, s);
					} else {
						s = fmt.Sprintf("%c[34;1m> %s%c[0m\n", 033, input.String(), 033);
						t.WriteAt(t.MaxRow-2, 0, s);
					}
					t.CursorAt(t.MaxRow-1, 0);
					input.Reset();
				}
			}
			n, _ = client.Read(&buff);
		}
	}
}

func cmdConnect(serverName string, client *ircclient.IRCClient, t *term.Term) {
	if client != nil {
		client.Connect(serverName, client.Nick);
	}
}

func cmdTLSConnect(serverName string, client *ircclient.IRCClient, t *term.Term) {
	if client != nil {
		client.TLSConnect(serverName, client.Nick);
	}
}

func cmdChannel(channel string, client *ircclient.IRCClient, t *term.Term) {
	if client != nil && len(channel) > 0 {
		chanParts := strings.Split(channel, " ", 2);
		client.SetChannel(chanParts[0]);
		if channel[0] < 'A' {
			client.Write(strings.Bytes("JOIN " + channel + "\r\n"));
		}
	}
}

func cmdAction(action string, client *ircclient.IRCClient, t *term.Term) {
	if (client != nil) {
		client.Write(strings.Bytes("PRIVMSG " + client.Channel + " :ACTION " + action + "\r\n"));
	}
}

func cmdQuit(reason string, client *ircclient.IRCClient, t *term.Term) {
	if client != nil {
		client.Write(strings.Bytes("QUIT\r\n"));
		client.Quit("user is done");
	}
	if t != nil {
		t.SetFullScroll();
		t.CursorAt(t.MaxRow, 0);
		t.ClearLine(t.MaxRow);
		t.Restore();
	}
	os.Exit(0);
}

func cmdHelp(topic string, client *ircclient.IRCClient, t *term.Term) {
	if t != nil {
		help, ok := topics[topic];
		if ok {
			t.WriteAt(t.MaxRow-2, 0, help);
		} else {
			s := fmt.Sprintf("%c[1m%s%s%s%s%s%s%s%s%s%s%s%s%c[0m\n", 033, "help\n",
			" /tlsconnect server:port - port for sirc typically 994\n",
			" /connect server:port - port for irc typically 6666-6669\n",
			" /join channel - join an irc channel, also used to make it the current active one\n",
			" /channel channel - same as join\n",
			" /part channel - leave a channel\n",
			" /me action - send an action to active channel\n",
			" /nick new-nick - change nick to new-nick\n",
			" /quit - disconnect from irc servers and exit program\n",
			" /resize - do this after changing the terminal size\n",
			" /whatever else - gets sent as-is to the server (basically raw command)\n",
			" text without a / - gets sent as a PRIVMSG to active channel\n", 033);
			t.WriteAt(t.MaxRow-2, 0, s);
		}
	}
}

func cmdResize(notused string, client *ircclient.IRCClient, t *term.Term) {
	if t != nil {
		t.SetFullScroll();
		t.Resize();
		t.SetScroll(0, t.MaxRow - 2);
		t.CursorAt(t.MaxRow - 1, 0);
	}
}
